/*
Scripts to handle interactivity in SVG topographic maps
Copyright (C) <2002>  <Andreas Neumann>, adapted by Dr. Olaf Schnabel 26-nov-2008
neumann@karto.baug.ethz.ch
http://www.carto.net/
http://www.karto.ethz.ch/neumann/

This javascript library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

----

original document site: http://www.carto.net/papers/svg/tuerlersee/
Please contact the author in case you want to use code or ideas commercially.
If you use this code, please include this copyright header, the included full
LGPL 2.1 text and read the terms provided in the LGPL 2.1 license
(http://www.gnu.org/copyleft/lesser.txt)
*/

var svgNS = "http://www.w3.org/2000/svg";

var sLselections0 = new Array("Rivers","Landcover","Contours");
var sLselections1 = new Array("Length","Object-Class","Name");
var sLselections2 = new Array("Area","Perimeter","Object-Class");
var sLselections3 = new Array("Length","Height");
var sLselections10 = new Array(0,10,20,30,40,50,60,70,80,90,100);
var sLselections11 = new Array("Villages","Mountains");
var sLselections12 = new Array(); //will be filled later --> see attribute_data.js/villages
var sLselections13 = new Array(); //will be filled later --> see attribute_data.js/mountains
var sLselections21 = new Array("Habersaat-Tuerlersee","Aeugst-Tuefenbach"); //routes

var statusObj;
var svgObj;
var svgStyle;
var zoomValueObj;
var mapAllWidth = 5137.6;
var mapAllHeight = 4000;
var mapHeight = 4000;
var mapWidth = 4000;
var xOriginCorner = 678431.2;
var yOriginCorner = -238000;
var pressed = 0;
var pluginPixWidth;
var pluginPixHeight;
var svgRect;
var svgMainViewport;
var width;
var height;
var zoomVal;
var currentID;
var evtX;
var evtY;
var rectUlXCorner;
var rectUlYCorner;
var scaleFactor;
var myYOffset;
var textXCoor;
var textYCoor;
var textZCoor;
var mapPixelWidth = 700;
var xnulcorner;
var ynulcorner;
var xulcorner;
var yulcorner;
var mapOffsetX;
var mapOffsetY;
var pixSize;
var activeLayer = 0;
var activeLayerName = sLselections0[activeLayer];
var activeAttribute = 0;
var myDataDisplayText;
//dtm basedata
var dtmCellSize = 25;
var xllcorner = 679000;
var yllcorner = 234000;
var nrows = 160;
var ncols = 160;
//profile variables
var profileStatus = "not started";
var profileClickStatus = 0;
var profXArray = new Array();
var profYArray = new Array();
var profDataStr;
var digitProfLine;
var profLineArrIndex;
var densify_distance = 20;
var profDensXArray = new Array();
var profDensYArray = new Array();
var profDensZArray = new Array();
var profileWidth = 667;
var profileHeight = 100;
var profZmax;
var profZmin;
var profileOffsetY = 5;
var profileOffsetX = 2;
var profileBottom = 745;
var profileLength;
var myProfileGroup;
var myProfileGridLineGroup;
var profileDrawn = 0;
var myDistanceText;
var myHeightText;
var myTextPath;
//settings for hillshade knobs
var elevKnobStatus = 0;
var elevValue = 45;
var elevKnob;
var azimKnobStatus = 0;
var azimKnob;
//vars for locating objects
var myBlinkNr = 0;
var activeSearchTopic = 0;
var activeSearchTopicName = sLselections11[activeSearchTopic];
//vars for locating routes
var indexRoute = 0;


//var layers=["Gridlines","Rivers","Landcover","relief","situation","Contours"];

function initMap() {
if (window.navigator) 
  {
  if ((!window.navigator.appName.match(/Adobe/gi)) || (!window.navigator.appName.match(/Opera/gi)))
      {
            var fakeScript = document.createElementNS("http://www.w3.org/2000/svg","script");
            fakeScript.setAttributeNS(null,"type","text/ecmascript");
            fakeScript.setAttributeNS("http://www.w3.org/1999/xlink","href","fake_smil.js");
            document.documentElement.appendChild(fakeScript);
      }
  }

                //initializing values
	zoomVal = 200; //initial zoomFactor
	//statusbar
	statusObj = document.getElementById("statusText").firstChild;
	//overviewObjects
	overviewViewport = document.getElementById("overviewMap");		
	pluginPixWidth = overviewViewport.getAttributeNS(null,"width");
	pluginPixHeight = overviewViewport.getAttributeNS(null,"height");
	//reference to draggable rectangle and viewport
	svgRect = document.getElementById("rectForPlace");
	svgMainViewport = document.getElementById("mainMap");
	zoomValueObj = document.getElementById("zoomValueObj").firstChild;
	//reference to coord-textfields
	textXCoor = document.getElementById("xcoor").firstChild;
	textYCoor = document.getElementById("ycoor").firstChild;
	textZCoor = document.getElementById("zcoor").firstChild;
	//reference to dataDisplay-Text
	myDataDisplayText = document.getElementById("dataDisplayText").firstChild;
	//references to profileElements
	myProfileGridLineGroup = document.getElementById("profileGridLineGroup");
	myDistanceText = document.getElementById("distanceText").firstChild;
	myHeightText = document.getElementById("heightText").firstChild;
	//for selectionLists
	resetFactors();
	//usage: createSelectionList(IndexToStringArray,width,xOffset,yOffset,nrToSelect,functionName);
	//create an empty group width Id "selection+indexNr"
	sLcreateSelectionList(3,140,880,340,3,0,"newVar"); //variables list
	sLcreateSelectionList(2,140,880,340,3,0,"newVar"); //variables list
	sLcreateSelectionList(1,140,880,340,3,0,"newVar"); //variables list
	sLcreateSelectionList(0,140,720,340,3,0,"newLayer"); //layer list
	sLcreateSelectionList(10,40,760,455,5,5,"landcoverTransp"); //transparency list
	sLcreateSelectionList(11,140,720,620,5,0,"searchLayer"); //names to search
	//villages
	for (i=0;i<15;i++) {
		sLselections12[i] = eval("Villages"+i)[0];
	}
	//mountains
	for (i=0;i<12;i++) {
		sLselections13[i] = eval("Mountains"+i)[0];
	}
	sLcreateSelectionList(12,140,880,620,4,0,"locateObject"); //layers to search
	sLcreateSelectionList(13,140,880,620,4,0,"locateObject"); //layers to search
	sLcreateSelectionList(21,235,720,570,2,0,"changeShowRoute"); //show routes
    //initialize zoomValues
    zoomItReally();
    statusChange("Map Loaded");
}

function resetFactors() {
	myRatio = 1024/768;
	if ((window.innerWidth/window.innerHeight) > myRatio) { //case window is more wide than myRatio
		scaleFactor = 768 / window.innerHeight;
	}
	else { //case window is more tall than myRatio
		scaleFactor = 1024 / window.innerWidth;		
	}
	mapOffsetX = (window.innerWidth - 1024 * 1 / scaleFactor) / 2;
	mapOffsetY = (window.innerHeight - 768 * 1 / scaleFactor) / 2;
}

function showCoords(evt) {
	//to show coordinates
	//statusChange("pixSize:"+pixSize+", xulcorner:"+xulcorner+", yulcorner:"+yulcorner+", mapOffsetX:"+Math.round(mapOffsetX)+", mapOffsetY:"+Math.round(mapOffsetY)+", clientX:"+evt.clientX+", clientY:"+evt.clientY+", scaleFactor:"+scaleFactor);
	xcoord = xnulcorner+(evt.clientX- mapOffsetX)*pixSize;
	ycoord = (ynulcorner+(evt.clientY- mapOffsetY)*pixSize)*-1;
	textXCoor.nodeValue=Math.round(xcoord);
	textYCoor.nodeValue=Math.round(ycoord);
	//calculate z
	//test if inside data-array
	if (((xcoord > xllcorner) && (xcoord < (xllcorner + mapWidth))) && ((ycoord > yllcorner) && (ycoord < (yllcorner + mapAllHeight)))) {
		//determine position within grid
		zval = getZval(xcoord,ycoord);
		textZCoor.nodeValue=Math.round(zval);
	}
}

function getZval(xcoord,ycoord) {
		row_ceil = nrows - Math.ceil((ycoord - yllcorner) / dtmCellSize);
		row_floor = nrows - Math.floor((ycoord - yllcorner) / dtmCellSize);	
		if (row_ceil < 1) { row_ceil = 1; }
		if (row_floor < 1) { row_floor = 1; }
		if (row_ceil > nrows) { row_ceil = nrows; }
		if (row_floor > nrows) { row_floor = nrows; }
		col_ceil = Math.ceil((xcoord - xllcorner) / dtmCellSize);
		col_floor = Math.floor((xcoord - xllcorner) / dtmCellSize);
		if (col_ceil < 0) { col_ceil = 0; }
		if (col_floor < 0) { col_floor = 0; }
		if (col_ceil > ncols - 1) { col_ceil = ncols - 1; }
		if (col_floor > ncols - 1) { col_floor = ncols -1; }
		xllcoord = xllcorner + col_floor * dtmCellSize;
		yllcoord = yllcorner + (nrows - row_floor) * dtmCellSize;
		zval = intBilinear(eval("dhm"+row_ceil)[col_ceil], eval("dhm"+row_ceil)[col_floor], eval("dhm"+row_floor)[col_floor], eval("dhm"+row_floor)[col_ceil], xcoord, ycoord, xllcoord, yllcoord, dtmCellSize);	
		return (zval);
}

//simulating statusbar
function statusChange(text) {
	statusObj.nodeValue = text;
}

function beginPan(evt) {
	pressed = 1;
	width = parseFloat(svgRect.getAttributeNS(null,"width"));
	height = parseFloat(svgRect.getAttributeNS(null,"height"));
	evtX = parseFloat(evt.clientX) * scaleFactor;
	evtY = parseFloat(evt.clientY) * scaleFactor;
	rectUlXCorner = parseFloat(svgRect.getAttributeNS(null,"x"));
	rectUlYCorner = parseFloat(svgRect.getAttributeNS(null,"y"));	
}

function doPan(evt) {
	if (pressed == 1) {
		newEvtX = parseFloat(evt.clientX) * scaleFactor; //scaleFactor is because of resizable interface
		newEvtY = parseFloat(evt.clientY) * scaleFactor;
		toMoveX = rectUlXCorner + (newEvtX - evtX) * mapAllWidth / pluginPixWidth;
		toMoveY = rectUlYCorner + (newEvtY - evtY) * mapAllHeight / pluginPixHeight;
		
		//restrict to borders of overviewmap
		if (toMoveX < xOriginCorner) {
			svgRect.setAttributeNS(null,"x",xOriginCorner);
		}
		else if ((toMoveX + width) > (xOriginCorner + mapAllWidth)) {
			svgRect.setAttributeNS(null,"x",xOriginCorner + mapAllWidth - width);				
		}
		else {
			svgRect.setAttributeNS(null,"x",toMoveX);			
		}
		if (toMoveY < yOriginCorner) {
			svgRect.setAttributeNS(null,"y",yOriginCorner);
		}
		else if ((toMoveY + height) > (yOriginCorner + mapAllHeight)) {
			svgRect.setAttributeNS(null,"y",yOriginCorner + mapAllHeight - height);
		}				
		else {
			svgRect.setAttributeNS(null,"y",toMoveY);
		}
		
		evtX = newEvtX;
		evtY = newEvtY;
		rectUlXCorner = parseFloat(svgRect.getAttributeNS(null,"x"));
		rectUlYCorner = parseFloat(svgRect.getAttributeNS(null,"y"));	
	}	
}

function endPan() {
	statusChange("panning map - please be patient ...");
	pressed = 0;
	//set viewport of main map
	xulcorner = parseFloat(svgRect.getAttributeNS(null,"x"));
	yulcorner = parseFloat(svgRect.getAttributeNS(null,"y"));
	xnulcorner = xulcorner;
	ynulcorner = yulcorner;
	width = parseFloat(svgRect.getAttributeNS(null,"width"));
	height = parseFloat(svgRect.getAttributeNS(null,"height"));
	newViewport = xulcorner + " " + yulcorner + " " + width + " " + height;
	svgMainViewport.setAttributeNS(null,"viewBox",newViewport);
	statusChange("map ready ...");
}

function zoomIt(inOrOut) {
	if (inOrOut == "in") {
		if (zoomVal < 1000) {
			statusChange("click to zoom in");
			zoomVal = zoomVal + 100;
			zoomItReally();
		}
		else {
			statusChange("maximum zoom factor reached! cannot zoom in any more!");
		}
	}
	if (inOrOut == "out") {
		if (zoomVal > 100) {
			statusChange("click to zoom out");
			zoomVal = zoomVal - 100;
			zoomItReally();
		}
		else {
			statusChange("minimum zoom factor reached! cannot zoom out any more!");			
		}		
	}
	if (inOrOut == "full") {
		if (zoomVal > 100) {
			statusChange("click to set to full-view");
			zoomVal = 100;
			zoomItReally();
		}
		else {
			statusChange("Full view already reached!");			
		}		
	}
}

function zoomItReally() {
	statusChange("panning map - please be patient ...");
	
	//get values from draggable rectangle
	xulcorner = parseFloat(svgRect.getAttributeNS(null,"x"));
	yulcorner = parseFloat(svgRect.getAttributeNS(null,"y"));
	width = parseFloat(svgRect.getAttributeNS(null,"width"));
	height = parseFloat(svgRect.getAttributeNS(null,"height"));
	
	//calcs
	xcenter = xulcorner + width / 2;
	ycenter = yulcorner + height / 2;
	xnulcorner = xcenter - mapAllWidth / 2 * (100/zoomVal);
	ynulcorner = ycenter - mapAllHeight / 2 * (100/zoomVal);
	nWidth = mapAllWidth * (100/zoomVal);
	nHeight = mapAllHeight * (100/zoomVal);
	pixSize = nWidth / (mapPixelWidth * 1 / scaleFactor);

	if (zoomVal == 100) {
		xnulcorner = xOriginCorner;
		ynulcorner = yOriginCorner;
	}		
	//set values of draggable rectangle
	svgRect.setAttributeNS(null,"x",xnulcorner);
	svgRect.setAttributeNS(null,"y",ynulcorner);
	svgRect.setAttributeNS(null,"width",nWidth);
	svgRect.setAttributeNS(null,"height",nHeight);
	
	//set viewport of main map
	newViewport = xnulcorner + " " + ynulcorner + " " + nWidth + " " + nHeight;
	svgMainViewport.setAttributeNS(null,"viewBox",newViewport);
	zoomValueObj.nodeValue = zoomVal;
	//set width/height for Scale-Line
	document.getElementById("myScaleTextH").firstChild.nodeValue = Math.round(nHeight)+" m";
	document.getElementById("myScaleTextW").firstChild.nodeValue = Math.round(nWidth)+" m";
	statusChange("map ready ...");
}

function newVar(myNumber) {
	//a new variable had been selected
	activeAttribute = myNumber;
}

function newLayer(myNumber,nrList) {
	//new layer had been selected
	activeLayerName = sLselections0[activeLayer];
	document.getElementById("sLselectionBox"+(activeLayer+1)).setAttributeNS(null,"visibility","hidden");
	if (activeLayerName == "Rivers" || activeLayerName == "Contours") {
		document.getElementById(activeLayerName).setAttributeNS(null,"pointer-events","none");
	}
	activeLayer = myNumber;
	activeLayerName = sLselections0[activeLayer];
	document.getElementById("sLselectionBox"+(activeLayer+1)).setAttributeNS(null,"visibility","visible");
	if (activeLayerName == "Rivers" || activeLayerName == "Contours") {
		document.getElementById(activeLayerName).setAttributeNS(null,"pointer-events","stroke");
	}
}

function showAttribData(evt,layer,objectId) {
	if (layer == activeLayerName) {
		if (evt.type == "mouseover") {
			myElement = layer+objectId;
			myString = eval(myElement)[activeAttribute];
			if (!myString) {
				myString = "-";
			}	
			myDataDisplayText.nodeValue=myString;		
		}
		if (evt.type == "mouseout") {
			myDataDisplayText.nodeValue="-";
		}
	}
}

function layerMenu(layerName) {
	myLayerObj = document.getElementById(layerName);
	myLayerVisibility = myLayerObj.getAttributeNS(null,"visibility");
	myCheckCrossObj = document.getElementById("checkCross"+layerName);
	if (myLayerVisibility == "visible") {
		myLayerObj.setAttributeNS(null,"visibility","hidden");
		myCheckCrossObj.setAttributeNS(null,"visibility","hidden");
	}
	else {
		myLayerObj.setAttributeNS(null,"visibility","visible");		
		myCheckCrossObj.setAttributeNS(null,"visibility","visible");
	}
}

function showAbout() {
		switchLayer("aboutMap","visible");
}

function switchLayer(layerName,visibility) {
	document.getElementById(layerName).setAttributeNS(null,"visibility",visibility);
}

function aboutClose(evt) {
	if (evt.type == "mouseover") {
		statusChange("click to close about-panel");
	}
	if (evt.type == "mouseout") {
		statusChange("map ready ...");
	}
	if (evt.type == "click") {
		switchLayer("aboutMap","hidden");
		statusChange("map ready ...");
	}	
}

//magnifier glass mouse-over effects
function magnify(evt,scaleFact,inOrOut) {
	if (inOrOut == "in") {
		if (zoomVal < 1000) {
			statusChange("click to zoom in");
			scaleObject(evt,scaleFact);
		}
		else {
			statusChange("maximum zoom factor reached! cannot zoom in any more!");			
		}
	}
	if (inOrOut == "out") {
		if (zoomVal > 100) {
			statusChange("click to zoom out");
			scaleObject(evt,scaleFact);
		}
		else {
			statusChange("minimum zoom factor reached! cannot zoom out any more!");			
		}		
	}
	if (inOrOut == "full") {
		if (zoomVal > 100) {
			statusChange("click to set full view");
			scaleObject(evt,scaleFact);
		}
		else {
			statusChange("full view already reached!");			
		}		
	}
	if (scaleFact == 1) {
			statusChange("map ready");		
			scaleObject(evt,scaleFact);
	}	
}

//scale any object that has a transform-value
function scaleObject(evt,factor) {
        //reference to the currently selected object
        var element = evt.currentTarget;

        //query old transform value (we need the translation value)
        var curTransform = element.getAttributeNS(null,"transform");
        curTransform = new String(curTransform); //Wert in ein String umwandeln
        //no fear from Regular expressions ... just copy it, I copied it either ...
        var translateRegExp=/translate\(([-+]?\d+)(\s*[\s,]\s*)([-+]?\d+)\)\s*/;

        //This part extracts the translation-value from the whole transform-string
        if (curTransform.length != 0)
        {
          var result = curTransform.match(translateRegExp);
          if (result == null || result.index == -1)
          {
             oldTranslateX = 0;
             oldTranslateY = 0;
          }
          else
          {
             oldTranslateX = result[1];
             oldTranslateY = result[3];
          }
          //concatenate the string again, add scale-factor
          var newtransform = "translate(" + oldTranslateX + " " + oldTranslateY + ") " + "scale(" + factor + ")";
        }
        //set transform-factor
        element.setAttributeNS(null,'transform', newtransform);
}

function checkBoxScript(evt) { //checkBox for toggling layers an contextMenue
	var element = evt.currentTarget;
	var text = new String(element.id);
	myLayerId = text.replace(/checkBox/,"");
	text = "checkCross"+myLayerId;
	myCheckCrossObj = document.getElementById(text);
	myCheckCrossVisibility = myCheckCrossObj.getAttributeNS(null,"visibility");
	if (evt.type == "mouseover") {
		if (myCheckCrossVisibility == "visible") {
			statusChange("click to turn off layer '"+myLayerId+"'...");
		}
		else {
			statusChange("click to turn on layer '"+myLayerId+"'...");
		}
	}
	if (evt.type == "mouseout") {
		statusChange("map ready ...");
	}
	if (evt.type == "click") {
		layerMenu(myLayerId);
	}
}

/* ------------------------- functions for interactive profile ---------------- */
/* ---------------------------------------------------------------------------- */
function startProfile(evt) { //start drawing profile line
	if (evt.type == "click") {
		document.getElementById("profileRect").setAttributeNS(null,"visibility","visible");
		startProfileRect = evt.target;
		startProfileRect.setAttributeNS(null,"style","fill:lightgray;stroke:darkcyan;stroke-width:2;");		
	}	
}

function drawProfileLine(evt) { //draw profile line in main map
	if (evt.type == "click") {
		if (profileClickStatus == 0) {
			profDataStr = "";
			profileClickStatus = 1;
			profLineArrIndex = 1;
			profXArray = [];
			profYArray = [];
			profXArray[0] = Math.round(xnulcorner+(evt.clientX- mapOffsetX)*pixSize);
			profYArray[0] = Math.round(ynulcorner+(evt.clientY- mapOffsetY)*pixSize);
			profDataStr = "M"+profXArray[0]+" "+profYArray[0];
			digitProfLine = document.createElementNS(svgNS,"path");
			digitProfLine.setAttributeNS(null,"id","digitProfLineA");
			digitProfLine.setAttributeNS(null,"style","fill:none;stroke:red;stroke-width:5;marker:url(#profileLineMarker);");
			digitProfLine.setAttributeNS(null,"d",profDataStr);
			document.getElementById("profileContainer").appendChild(digitProfLine);
		}
		else {
			profXArray[profLineArrIndex] = Math.round(xnulcorner+(evt.clientX- mapOffsetX)*pixSize);
			profYArray[profLineArrIndex] = Math.round(ynulcorner+(evt.clientY- mapOffsetY)*pixSize);
			profDataStr = profDataStr+"L "+profXArray[profLineArrIndex]+" "+profYArray[profLineArrIndex];
			digitProfLine.setAttributeNS(null,"d",profDataStr);
			if (profLineArrIndex == 1) {
				profileStatus = "line drawn";	
			}
			profLineArrIndex++;
		}
	}	
}

function removeProfile(evt) { //remove profile line in main map and profile group
	if (evt.type == "click") {
		if (profileStatus == "not started") {
			alert("no profile drawn - cannot remove anything ...");
		}
		else {
			document.getElementById("profileContainer").removeChild(digitProfLine);
			document.getElementById("startProfileRect").setAttributeNS(null,"style","fill:white;stroke:darkcyan;stroke-width:2;");		
			if (profileDrawn == 1) {
				myProfGroup = document.getElementById("myProfileGroup");
				myProfileGroup.removeChild(myProfGroup);
				myGridGroup = document.getElementById("myGridGroup");
				myProfileGridLineGroup.removeChild(myGridGroup);
				myMarkerText = document.getElementById("textOnProfile");
				document.getElementById("profileContainer").removeChild(myMarkerText);		
			}
			myProfileGridLineGroup.setAttributeNS(null,"visibility","hidden");
			//re-set button to white
			document.getElementById("createProfileRect").setAttributeNS(null,"style","fill:white;stroke:darkcyan;stroke-width:2;");		
			//re-set status variables
			profileClickStatus = 0;
			profileDrawn = 0;
			profileStatus = "not started";
		}			
	}	
}

function createProfile(evt) { //create profile geometry
	if (evt.type == "click") {
		if (profileStatus == "line drawn") {
		//set button to gray
		document.getElementById("createProfileRect").setAttributeNS(null,"style","fill:lightgray;stroke:darkcyan;stroke-width:2;");		
		document.getElementById("profileRect").setAttributeNS(null,"visibility","hidden");		
		
		myProfileGridLineGroup.setAttributeNS(null,"visibility","visible");

		profileLength = digitProfLine.getTotalLength();
		densify_distance = profileLength / (profileWidth / 2);
      		nr_points = Math.floor(profileLength /densify_distance); //calculates number points according to the densify distance
		
		length_new = densify_distance;
		i = 1;
		count_newfield = 0; //counter for newly calculated array
		dist_incr = toPolarDist((profXArray[1]-profXArray[0]),(profYArray[1]-profYArray[0]));
		direction = toPolarDir((profXArray[1]-profXArray[0]),(profYArray[1]-profYArray[0]));
		profDensXArray[0] = profXArray[0];
		profDensYArray[0] = profYArray[0];
		profDensZArray[0] = getZval(profDensXArray[0],profDensYArray[0]);
		profZmax = profDensZArray[0];	
		profZmin = profDensZArray[0];
		var profZmean = 0;

                            //densify coordinates according to densify-distance
		while(length_new < profileLength) {
			count_newfield++;
			xincr = toRectX(direction,densify_distance);
			yincr = toRectY(direction,densify_distance);
			profDensXArray[count_newfield] = profDensXArray[count_newfield-1] + xincr;
			profDensYArray[count_newfield] = profDensYArray[count_newfield-1] + yincr;
			profDensZArray[count_newfield] = getZval(profDensXArray[count_newfield],profDensYArray[count_newfield]*-1);
			if (profDensZArray[count_newfield] > profZmax) {
				profZmax = profDensZArray[count_newfield];
			}
			if (profDensZArray[count_newfield] < profZmin) {
				profZmin = profDensZArray[count_newfield];
			}
			profZmean = profZmean + profDensZArray[count_newfield];

			//this if statement is to change direction when the next vertice from original line comes
			if ((length_new > (dist_incr - densify_distance)) && (i < profXArray.length - 1 )) {
           			count_newfield++;
				tempdist = dist_incr - length_new;
 				length_new = length_new + tempdist;
				//guess if next segment_length longer than tempdist
				//this has still to be done ...
				xtemp = profXArray[i];
				ytemp = profYArray[i];
				tempnewdist = densify_distance - tempdist;
				direction = toPolarDir((profXArray[i+1]-profXArray[i]),(profYArray[i+1]-profYArray[i]));
				xincr = toRectX(direction,tempnewdist);
				yincr = toRectY(direction,tempnewdist);
				profDensXArray[count_newfield] = xtemp + xincr;
				profDensYArray[count_newfield] = ytemp + yincr;
				profDensZArray[count_newfield] = getZval(profDensXArray[count_newfield],profDensYArray[count_newfield]*-1);
				if (profDensZArray[count_newfield] > profZmax) {
					profZmax = profDensZArray[count_newfield];
				}
				if (profDensZArray[count_newfield] < profZmin) {
					profZmin = profDensZArray[count_newfield];
				}
				distance = toPolarDist((profXArray[i+1]-profXArray[i]),(profYArray[i+1]-profYArray[i]));
           			length_new = length_new + tempnewdist;

           			dist_incr = dist_incr + distance;
				i++;
	   		}
	   		length_new = length_new + densify_distance;	
		}
		profZmax = Math.round(profZmax);
		profZmin = Math.round(profZmin);
		profZmean = Math.round(profZmean / nr_points);
		
		//now create profile
		oneElementWidth = profileWidth / count_newfield;
		myMeterHeight = (profileHeight - profileOffsetY) / (profZmax - profZmin); //ensure a minimum height
		//references to group-elements
		myProfileGroup = document.getElementById("profile");
		myProfileLineGroup = document.getElementById("profileGridLineGroup");
		profileX = profileOffsetX;
		myProfGroup = document.createElementNS(svgNS,"g");
		myProfGroup.setAttributeNS(null,"id","myProfileGroup");
		myProfGroup.setAttributeNS(null,"style","fill:mediumturquoise;stroke:none;");
		myProfileGroup.appendChild(myProfGroup);
		//create interactive rects
		myD = "M";
		for (i = 1; i <= count_newfield; i++) {
			myValue = profDensZArray[i];
			mySliceHeight = profileOffsetY + (myValue - profZmin) * myMeterHeight;
			myProfSlice=document.createElementNS(svgNS,"rect");
			myProfSlice.setAttributeNS(null,"x",profileX);
			myProfSlice.setAttributeNS(null,"y",profileBottom-mySliceHeight);
			myProfSlice.setAttributeNS(null,"width",oneElementWidth+0.5);
			myProfSlice.setAttributeNS(null,"height",mySliceHeight);
			//concatenate d attribute for path
			myD = myD+parseInt(profileX+oneElementWidth/2)+" "+parseInt(profileBottom-mySliceHeight)+"L";
			//add events
			myProfSlice.setAttributeNS(null,"onmouseover","showProfileData(evt,"+myValue+","+i+","+densify_distance+");");
			myProfSlice.setAttributeNS(null,"onmouseout","showProfileData(evt,"+myValue+","+i+","+densify_distance+");");
			myProfSlice.setAttributeNS(null,"onclick","showProfileData(evt,"+myValue+","+i+","+densify_distance+");");
			myProfGroup.appendChild(myProfSlice);
			profileX = profileX + oneElementWidth;
		}

                               //create profileLine above
		myD=myD+profileX+" "+parseInt(profileBottom-mySliceHeight);
		myClipD = myD+"L"+parseInt(profileX)+" "+parseInt(profileBottom)+"L"+parseInt(profileOffsetX)+" "+parseInt(profileBottom)+"z";
		myProfileLineAbove = document.createElementNS(svgNS,"path");
		myProfileLineAbove.setAttributeNS(null,"style","stroke:darkcyan;stroke-width:2;fill:none;");
		myProfileLineAbove.setAttributeNS(null,"d",myD);
		myClipPath = document.createElementNS(svgNS,"clipPath");
		myClipPath.setAttributeNS(null,"id","profileClip");
		myClipPathGeometry = document.createElementNS(svgNS,"path");
		myClipPathGeometry.setAttributeNS(null,"d",myClipD);
		myClipPathGeometry.setAttributeNS(null,"style","stroke:none;fill:red;");
		myClipPathGeometry.setAttributeNS(null,"opacity","0.5");
		myClipPath.appendChild(myClipPathGeometry);
		
		myGridLineGroup = document.getElementById("profileGridLineGroup");
		//create horizontalgridlines
		myGridGroup = document.createElementNS(svgNS,"g");
		myGridGroup.setAttributeNS(null,"id","myGridGroup");
		myGridLineGroup.appendChild(myGridGroup);
		//append clipPath
		myGridGroup.appendChild(myClipPath);
		myInterval = 50;	
		if ((profZmax - profZmin) > 300) {
			myInterval = 100;	
		}
		if ((profZmax - profZmin) < 100) {
			myInterval = 20;	
		}
		if ((profZmax - profZmin) < 50) {
			myInterval = 10;	
		}
		myYval = Math.ceil(profZmin / myInterval) * myInterval;
		var myRectBottom = profileBottom;
		var i = 0;
		var myGrayVal = 20;
		var myGrayIncr = Math.ceil(235 / ((profZmax - profZmin) / myInterval));
		while(myYval < profZmax) {
			myYcoor = profileBottom - (profileOffsetY + (myYval - profZmin) * myMeterHeight);
			//gray rects
			myTranspRect = document.createElementNS(svgNS,"rect");
			myTranspRect.setAttributeNS(null,"x",profileOffsetX);
			myTranspRect.setAttributeNS(null,"y",myYcoor);
			myTranspRect.setAttributeNS(null,"width",profileWidth);
			myTranspRect.setAttributeNS(null,"height",myRectBottom - myYcoor);
			myTranspRect.setAttributeNS(null,"style","fill:rgb("+myGrayVal+","+myGrayVal+","+myGrayVal+");stroke:none;");
			myTranspRect.setAttributeNS(null,"opacity","0.5");
			myTranspRect.setAttributeNS(null,"pointer-events","none");
			myTranspRect.setAttributeNS(null,"clip-path","url(#profileClip)");
			myGridGroup.appendChild(myTranspRect);
			myRectBottom = myYcoor;
			myGrayVal = myGrayVal + myGrayIncr;
			//gridLines
			myGridLine=document.createElementNS(svgNS,"line");
			myGridLine.setAttributeNS(null,"x1",profileOffsetX-oneElementWidth);
			myGridLine.setAttributeNS(null,"y1",myYcoor);
			myGridLine.setAttributeNS(null,"x2",profileOffsetX+profileWidth+oneElementWidth);
			myGridLine.setAttributeNS(null,"y2",myYcoor);
			myGridLine.setAttributeNS(null,"style","stroke:darkcyan;stroke-width:0.5;");
			myGridLine.setAttributeNS(null,"id","gridLine"+i);
			myGridLine.setAttributeNS(null,"pointer-events","none");
			myGridGroup.appendChild(myGridLine);
			//text left
			myGridText=document.createElementNS(svgNS,"text");
			myGridText.setAttributeNS(null,"x",profileOffsetX+profileWidth+oneElementWidth*2);
			myGridText.setAttributeNS(null,"y",myYcoor+3);
			myGridText.setAttributeNS(null,"style","font-size:10px;font-family:Arial,Helvetica,sans-serif;fill:darkcyan;");
			myGridTextChild=document.createTextNode(myYval+" m");
			myGridText.appendChild(myGridTextChild);
			myGridText.setAttributeNS(null,"id","gridLineTextRight"+i);
			myGridGroup.appendChild(myGridText);
			//increase values
			myYval = myYval + myInterval;
			i = i + 1;
		}
		//last rect
		myYcoor = profileBottom - (profileOffsetY + (myYval - profZmin) * myMeterHeight);
		myTranspRect = document.createElementNS(svgNS,"rect");
		myTranspRect.setAttributeNS(null,"x",profileOffsetX);
		myTranspRect.setAttributeNS(null,"y",myYcoor);
		myTranspRect.setAttributeNS(null,"width",profileWidth);
		myTranspRect.setAttributeNS(null,"height",myRectBottom - myYcoor);
		myTranspRect.setAttributeNS(null,"style","fill:rgb("+myGrayVal+","+myGrayVal+","+myGrayVal+");stroke:none;");
		myTranspRect.setAttributeNS(null,"opacity","0.5");
		myTranspRect.setAttributeNS(null,"pointer-events","none");
		myTranspRect.setAttributeNS(null,"clip-path","url(#profileClip)");
		myGridGroup.appendChild(myTranspRect);
		myRectBottom = myYcoor;
		//append profileLine and rects to be clipped
		myGridGroup.appendChild(myProfileLineAbove);
		
		//create markertext to show position along profile line
		myMarkerText=document.createElementNS(svgNS,"text");
		myMarkerText.setAttributeNS(null,"id","textOnProfile");
		myMarkerText.setAttributeNS(null,"pointer-events","none");
		myMarkerText.setAttributeNS(null,"style","fill:red;font-family:Arial,Helvetica,sans-serif;font-size:80px;font-weight:bold;text-anchor:middle;");
		myTextPath = document.createElementNS(svgNS,"textPath");
		myTextPath.setAttributeNS("http://www.w3.org/1999/xlink","href","#digitProfLineA");
		myTextPath.setAttributeNS(null,"startOffset","0%");
		myMarkerText.setAttributeNS(null,"dy","20");
		myMarkerText.appendChild(myTextPath);
		myMarkerTextText=document.createTextNode("|");
		myTextPath.appendChild(myMarkerTextText);
		document.getElementById("profileContainer").appendChild(myMarkerText);
		//setStats
		document.getElementById("profileLengthText").firstChild.nodeValue=Math.round(profileLength)+" m";
		document.getElementById("profileMinText").firstChild.nodeValue=profZmin+" m";
		document.getElementById("profileMaxText").firstChild.nodeValue=profZmax+" m";
		document.getElementById("profileMeanText").firstChild.nodeValue=profZmean+" m";

		profileDrawn = 1;
                             profileStatus = "profile drawn";
	}
	else {
		if (profileStatus == "profile drawn") {
			alert("Please remove old profile first");
		}
		if (profileStatus == "not started") {
			alert("Please draw profile line on main map first - with at least two vertices");
		}
	}
	}
}

function showProfileData(evt,heightVal,nrSlice,oneElementWidth) { //zo show height, distance and move slider along profileLine
	myProfileSlice = evt.target;
	if (evt.type == "mouseover") {
		myHeightText.nodeValue=Math.round(heightVal)+" m";	
		myDistanceText.nodeValue=Math.round(nrSlice*oneElementWidth)+" m";
		myProfileSlice.setAttributeNS(null,"style","fill:yellow;stroke:none;");
		myPosition = (nrSlice*oneElementWidth - oneElementWidth*0.25)/profileLength*100;
		if (myPosition <= 0) {
			myPosition = 0;
		}
		if (myPosition >= 100) {
			myPosition = 100;	
		}
		myTextPath.setAttributeNS(null,"startOffset",myPosition+"%");
	}
	if (evt.type == "mouseout") {
		myProfileSlice.setAttributeNS(null,"style","fill:mediumturquoise;stroke:none;");			
	}
}

/* ----------------------- helper functions to calculate stuff ---------------- */
/* ---------------------------------------------------------------------------- */
function toPolarDir(xdiff,ydiff) { // Subroutine for calculating polar Coordinates
   direction = (Math.atan2(ydiff,xdiff));
   return(direction);
}

function toPolarDist(xdiff,ydiff) { // Subroutine for calculating polar Coordinates
   distance = Math.sqrt(xdiff * xdiff + ydiff * ydiff);            
   return(distance);
}

function toRectX(direction,distance) { // Subroutine for calculating cartesic coordinates

   x = distance * Math.cos(direction);
   y = distance * Math.sin(direction);
          
   return(x);
}

function toRectY(direction,distance) { // Subroutine for calculating cartesic coordinates

   x = distance * Math.cos(direction);
   y = distance * Math.sin(direction);
          
   return(y);
}

function intBilinear(za,zb,zc,zd,xpos,ypos,ax,ay,cellsize) { //bilinear interpolation function
	e = (xpos - ax) / cellsize;
	f = (ypos - ay) / cellsize;

	//calculation of weights
	wa = (1 - e) * (1 - f);
	wb = e * (1 - f);
	wc = e * f;
	wd = f * (1 - e);

	height_interpol = wa * zc + wb * zd + wc * za + wd * zb;

	return (height_interpol);	
}


function setElevation(evt) { //for interactive knob to set elevation angle
	if (evt.type=="mousedown") {
		elevKnobStatus = 1;
		elevKnob = document.getElementById("elevLine");
	}
	if (evt.type=="mousemove") {
		if (elevKnobStatus == 1) {
			myXdiff = 990 - (evt.clientX - mapOffsetX) * scaleFactor;
			myYdiff = 475 - (evt.clientY - mapOffsetY) * scaleFactor;
			direction = (180 - (toPolarDir(myXdiff,myYdiff) / 3.14 * 180))*-1;
			if ((direction < -90) && (direction > -180)) {
				elevKnob.setAttributeNS(null,"transform","rotate("+direction+",990,520)");
				elevValue = 180 + direction;
			}
		}
	}
	if (evt.type=="mouseup") {
		elevKnobStatus = 0;
		myLightSource = document.getElementById("lightsource");
		myLightSource.setAttributeNS(null,"elevation",elevValue);
	}
}

function setAzimuth(evt) { //for interactive knob to set elevation angle
	if (evt.type=="mousedown") {
		azimKnobStatus = 1;
		azimKnob = document.getElementById("azimLine");
	}
	if (evt.type=="mousemove") {
		if (azimKnobStatus == 1) {
			myXdiff = 900 - (evt.clientX - mapOffsetX) * scaleFactor;
			myYdiff = 475 - (evt.clientY - mapOffsetY) * scaleFactor;
			direction = (180 - (toPolarDir(myXdiff,myYdiff) / 3.14 * 180))*-1;
			azimKnob.setAttributeNS(null,"transform","rotate("+direction+",900,520)");
			azimValue = direction;
			if ((direction < 0) && (direction > -90)) { 
				azimValue = direction + 90;
			}
			if ((direction > -360) && (direction < -180)) {
				azimValue = 360 + direction + 90;
			}
			if ((direction < -90) && (direction > -180)) { 
				azimValue = 270 + 180 + direction;
			}
		}
	}
	if (evt.type=="mouseup") {
		azimKnobStatus = 0;
		myLightSource = document.getElementById("lightsource");
		myLightSource.setAttributeNS(null,"azimuth",direction);
	}
}

function landcoverTransp(nrSelect) {
	myLandcover = document.getElementById("Landcover");
	myLandcover.setAttributeNS(null,"style","fill-opacity:"+nrSelect/10+";");
}

/* ----------------------- geographically locate objects ---------------------- */
/* ---------------------------------------------------------------------------- */
function searchLayer(nrElement,nrList) { //toggle selectionLists for different topics
	activeSearchTopicName = sLselections11[activeSearchTopic];
	document.getElementById("sLselectionBox"+(activeSearchTopic+12)).setAttributeNS(null,"visibility","hidden");
	activeSearchTopic = nrElement;
	document.getElementById("sLselectionBox"+(activeSearchTopic+12)).setAttributeNS(null,"visibility","visible");
}

function locateObject(nrElement,nrList) { //locate objects and set corresponding viewBox
	if (nrList == 12) {
		var myXmin = eval('Villages'+nrElement)[1];
		var myYmin = eval('Villages'+nrElement)[2];
		var myXmax = eval('Villages'+nrElement)[3];
		var myYmax = eval('Villages'+nrElement)[4];
	}
	else if (nrList == 13) {
		var myXmin = eval('Mountains'+nrElement)[1];
		var myYmin = eval('Mountains'+nrElement)[2];
		var myXmax = eval('Mountains'+nrElement)[3];
		var myYmax = eval('Mountains'+nrElement)[4];
	}
	//see if more width or more height
	var myWidth = myXmax - myXmin;
	var myHeight = myYmax - myYmin;
	if ((myWidth / myHeight) > (mapAllWidth / mapAllHeight)) { //case more wide
		var myFactor = Math.floor(mapAllWidth / myWidth);
	}
	else { //case more high
		var myFactor = Math.floor(mapAllHeight / myHeight);		
	}
	if (myFactor > 10) {
		myFactor = 10;	
	}
	//calculate actual new ViewBoxRect
	var myNewWidth = mapAllWidth / myFactor;
	var myNewHeight = mapAllHeight / myFactor;
	myX = myXmin + myWidth / 2 - myNewWidth / 2;
	myY = (myYmax - myHeight / 2 + myNewHeight / 2) * -1;
	//set rect in overviewmap
	svgRect.setAttributeNS(null,"x",myX);
	svgRect.setAttributeNS(null,"y",myY);
	svgRect.setAttributeNS(null,"width",myNewWidth);
	svgRect.setAttributeNS(null,"height",myNewHeight);
	zoomVal = myFactor * 100;
    zoomItReally();
}

function panOver(evt) {
	var myRect = evt.target;
	if (evt.type == "mouseover") {
		myRect.setAttributeNS(null,"fill","lightgray");
	}
	if (evt.type == "mouseout") {
		myRect.setAttributeNS(null,"fill","white");
	}
}

function pan(myX,myY,howmuch) {
	//get values from draggable rectangle
	xulcorner = parseFloat(svgRect.getAttributeNS(null,"x"));
	yulcorner = parseFloat(svgRect.getAttributeNS(null,"y"));
	width = parseFloat(svgRect.getAttributeNS(null,"width"));
	height = parseFloat(svgRect.getAttributeNS(null,"height"));

	//set values of draggable rectangle
	rectXulcorner = xulcorner + howmuch * width * myX;
	rectYulcorner = yulcorner + howmuch * height * myY;
	svgRect.setAttributeNS(null,"x",rectXulcorner);
	svgRect.setAttributeNS(null,"y",rectYulcorner);

	//set viewport of main map
	var newViewport = rectXulcorner + " " + rectYulcorner + " " + width + " " + height ;
	svgMainViewport.setAttributeNS(null,"viewBox",newViewport);

	//reset map boundary values	
	xcenter = rectXulcorner + width / 2;
	ycenter = rectYulcorner + height / 2;
	xnulcorner = xcenter - mapAllWidth / 2 * (100/zoomVal);
	ynulcorner = ycenter - mapAllHeight / 2 * (100/zoomVal);

	statusChange("map ready ...");
}

//show Extent of map viewBox
function showExtent() {
	with(svgRect) {
		alert("Xmin="+getAttributeNS(null,"x")+"m; Xmax="+(parseFloat(getAttributeNS(null,"x"))+parseFloat(getAttributeNS(null,"width")))+"m; Ymin="+((parseFloat(getAttributeNS(null,"y"))+parseFloat(getAttributeNS(null,"height")))*-1)+"m; Ymax="+(getAttributeNS(null,"y")*-1)+"m\nWidth="+getAttributeNS(null,"width")+"m; Height="+getAttributeNS(null,"height")+"m");
	}
}

// workaround for firefox bug (browser cannot execute function getElementById if the element animate is unknown)
// thanks to Frank Bruder for this function
function myGetElementById(id) 
{
  var element = document.getElementById(id);
  if(element==null && document.evaluate!=null) 
  {
    element = document.evaluate(
    '//*[@xml:id="' + id + '" or @id="' + id + '"]', document, function(ns) 
    {
        switch(ns) {
            case "xml": return "http://www.w3.org/XML/1998/namespace";
            case null: return null;
        }
   }, XPathResult.ANY_TYPE, null).iterateNext();
 }
 return element;
}

function showRoute(evt) {
	var myRoute = document.getElementById("route"+indexRoute);
	var myRouteLength = myRoute.getTotalLength();
	var myRouteAnim = myGetElementById("routeAnim"+indexRoute);
	//set animation parameters
	myRoute.setAttributeNS(null,"stroke-dasharray",myRouteLength.toString()+","+myRouteLength.toString());
	myRoute.setAttributeNS(null,"stroke-dashoffset",myRouteLength.toString());
	myRouteAnim.setAttributeNS(null,"from",myRouteLength.toString());
	myRouteAnim.setAttributeNS(null,"values",myRouteLength.toString()+";0");
		
	var myXmin = parseFloat(myRoute.getBBox().x);
	var myYmax = parseFloat(myRoute.getBBox().y) * -1;
	var myXmax = myXmin + parseFloat(myRoute.getBBox().width);
	var myYmin = (myRoute.getBBox().y + parseFloat(myRoute.getBBox().height)) * -1;
	
	//see if more width or more height
	var myWidth = myXmax - myXmin;
	var myHeight = myYmax - myYmin;
	if ((myWidth / myHeight) > (mapAllWidth / mapAllHeight)) { //case more wide
		var myFactor = Math.floor(mapAllWidth / myWidth);
	}
	else { //case more high
		var myFactor = Math.floor(mapAllHeight / myHeight);		
	}
	if (myFactor > 10) {
		myFactor = 10;	
	}
	//calculate actual new ViewBoxRect
	var myNewWidth = mapAllWidth / myFactor;
	var myNewHeight = mapAllHeight / myFactor;
	myX = myXmin + myWidth / 2 - myNewWidth / 2;
	myY = (myYmax - myHeight / 2 + myNewHeight / 2) * -1;
	//set rect in overviewmap
	svgRect.setAttributeNS(null,"x",myX);
	svgRect.setAttributeNS(null,"y",myY);
	svgRect.setAttributeNS(null,"width",myNewWidth);
	svgRect.setAttributeNS(null,"height",myNewHeight);
	zoomVal = myFactor * 100;

zoomItReally();

    window.setTimeout("routeStartAnim(indexRoute)",2000);
    window.setTimeout("routeSetVisBack('route'+indexRoute)",15000);
}

function changeShowRoute(nrElement,nrList) { //change SelectedRoute
	indexRoute = nrElement;
}

function routeStartAnim(myRouteNr)  {
	document.getElementById("route"+myRouteNr).setAttributeNS(null,"visibility","visible");
	var myRouteAnim = myGetElementById("routeAnim"+myRouteNr);
	myRouteAnim.beginElement();
}

function routeSetVisBack(myRoute)  {
	document.getElementById(myRoute).setAttributeNS(null,"visibility","hidden");
}
