/* ----------------------- helper functions to calculate stuff ---------------- */
/* ---------------------------------------------------------------------------- */
function toPolarDir(xdiff,ydiff) { // Subroutine for calculating polar Coordinates
   direction = (Math.atan2(ydiff,xdiff));
   //result is angle in radian
   return(direction);
}

function toPolarDist(xdiff,ydiff) { // Subroutine for calculating polar Coordinates
   distance = Math.sqrt(xdiff * xdiff + ydiff * ydiff);
   return(distance);
}

function toRectX(direction,distance) { // Subroutine for calculating cartesic coordinates
   x = distance * Math.cos(direction);
   y = distance * Math.sin(direction);
   return(x);
}

function toRectY(direction,distance) { // Subroutine for calculating cartesic coordinates
   x = distance * Math.cos(direction);
   y = distance * Math.sin(direction);
   return(y);
}

//test if point is left of or right of, result is 1 (leftof) or 0 (rightof)
function leftOfTest(pointx,pointy,linex1,liney1,linex2,liney2) {
	result = (liney1 - pointy) * (linex2 - linex1) - (linex1 - pointx) * (liney2 - liney1);
	if (result < 0) {
		leftof = 1; //case left of
	}
	else {
		leftof = 0; //case left of	
	}
	return(leftof);
}

//input is point coordinate, and 2 line coordinates
function distFromLine(xpoint,ypoint,linex1,liney1,linex2,liney2) {
	dx = linex2 - linex1;
	dy = liney2 - liney1;
	distance = (dy * (xpoint - linex1) - dx * (ypoint - liney1)) / Math.sqrt(Math.pow(dx,2) + Math.pow(dy,2));
	return(distance);
}

//converts radian value to degrees
function radian2deg(radian) {
	deg = radian / Math.PI * 180;
	return(deg);
}

//input is two vectors (a1,a2 is vector a, b1,b2 is vector b), output is angle in radian
//Formula: Acos from  Scalaproduct of the two vectors divided by ( norm (deutsch Betrag) vector 1 by norm vector 2
//see http://www.mathe-online.at/mathint/vect2/i.html#Winkel
function angleBetwTwoLines(a1,a2,b1,b2) {
	angle = Math.acos((a1 * b1 + a2 * b2) / (Math.sqrt(Math.pow(a1,2) + Math.pow(a2,2)) * Math.sqrt(Math.pow(b1,2) + Math.pow(b2,2))));
	return(angle);
}

//input is two vectors (a1,a2 is vector a, b1,b2 is vector b), output is new vector c2 returned as array
//Formula: Vektor a divided by Norm Vector a (Betrag) plus Vektor b divided by Norm Vector b (Betrag)
//see http://www.mathe-online.at/mathint/vect1/i.html#Winkelsymmetrale
function calcBisectorVector(a1,a2,b1,b2) {
	betraga = Math.sqrt(Math.pow(a1,2) + Math.pow(a2,2));
	betragb = Math.sqrt(Math.pow(b1,2) + Math.pow(b2,2));
	c = new Array();
	c[0] = a1 / betraga + b1 / betragb;
	c[1] = a2 / betraga + b2 / betragb;
	return(c);
}

//input is two vectors (a1,a2 is vector a, b1,b2 is vector b), output is angle in radian
//Formula: Vektor a divided by Norm Vector a (Betrag) plus Vektor b divided by Norm Vector b (Betrag)
//see http://www.mathe-online.at/mathint/vect1/i.html#Winkelsymmetrale
function calcBisectorAngle(a1,a2,b1,b2) {
	betraga = Math.sqrt(Math.pow(a1,2) + Math.pow(a2,2));
	betragb = Math.sqrt(Math.pow(b1,2) + Math.pow(b2,2));
	c1 = a1 / betraga + b1 / betragb;
	c2 = a2 / betraga + b2 / betragb;
	angle = toPolarDir(c1,c2);
	return(angle);
}

/* ----------------------- helper function to sort arrays ---------------- */
/* ----------------------------------------------------------------------- */
//my own sort function, uses only first part of string (population value)
function mySort(a,b) {
	var myResulta = a.split("+");
	var myResultb = b.split("+");
	if (parseFloat(myResulta[0]) < parseFloat(myResultb[0])) {
		return 1;
	}
	else {
		return -1;
	}
}

/* ----------------------- helper function format number strings -------------- */
/* ---------------------------------------------------------------------------- */
//formatting number strings
//this function add's "'" to a number every third digit
function formatNumberString(myString) {
	var myNewString="";
	var myCounter= myString.length;
	if (myString.length > 3) {
		while(myCounter > 0) {
			if (myCounter > 3) {
				myNewString = "," + myString.substr(myCounter - 3,3) + myNewString;
			}
			else {
				myNewString = myString.substr(0,myCounter) + myNewString;
			}
			myCounter -= 3;
		}
	}
	else {
		myNewString = myString;
	}
	return myNewString;
}

//function for status Bar
function statusChange(statusText) {
	document.getElementById("Help_Text").firstChild.nodeValue = "Statusbar: " + statusText;
}

//scale an object
function scaleObject(evt,factor) {
//reference to the currently selected object
	var element = evt.currentTarget;
	var myX = element.getAttributeNS(null,"x");
	var myY = element.getAttributeNS(null,"y");
	var newtransform = "scale(" + factor + ") translate(" + (myX * 1 / factor - myX) + " " + (myY * 1 / factor - myY) +")";
	element.setAttributeNS(null,'transform', newtransform);
	//statusChange(newtransform);
}

//calculate HSV 2 RGB: HSV (h 0 to 360, sat and val are between 0 and 1), RGB between 0 and 255
function hsv2rgb(hue,sat,val) {
	var rgbArr = new Array();
	if ( sat == 0) {
		rgbArr["red"] = Math.round(val * 255);
		rgbArr["green"] = Math.round(val * 255);
		rgbArr["blue"] = Math.round(val * 255);
	}
	else {
		var h = hue / 60;
		var i = Math.floor(h);
		var f = h - i;
		if (i % 2 == 0) {
			f = 1 - f;
		}
		var m = val * (1 - sat); 
		var n = val * (1 - sat * f);
		switch(i) {
			case 0:
				rgbArr["red"] = val;
				rgbArr["green"] = n;
				rgbArr["blue"] = m;
				break;
			case 1:
				rgbArr["red"] = n;
				rgbArr["green"] = val;
				rgbArr["blue"] = m;
				break;
			case 2:
				rgbArr["red"] = m;
				rgbArr["green"] = val;
				rgbArr["blue"] = n;
				break;
			case 3:
				rgbArr["red"] = m;
				rgbArr["green"] = n;
				rgbArr["blue"] = val;
				break;
			case 4:
				rgbArr["red"] = n;
				rgbArr["green"] = m;
				rgbArr["blue"] = val;
				break;
			case 5:
				rgbArr["red"] = val;
				rgbArr["green"] = m;
				rgbArr["blue"] = n;
				break;
			case 6:
				rgbArr["red"] = val;
				rgbArr["green"] = n;
				rgbArr["blue"] = m;
				break;
		}
		rgbArr["red"] = Math.round(rgbArr["red"] * 255);
		rgbArr["green"] = Math.round(rgbArr["green"] * 255);
		rgbArr["blue"] = Math.round(rgbArr["blue"] * 255);
	}
	return rgbArr;
}

function replaceSpecialChars(myString) {
		for (i=161;i<256;i++) {
			re = new RegExp("&#"+i+";","g");
			myString = myString.replace(re,String.fromCharCode(i));
		}
		return myString;
}