/******************************************************************************************/
//global variables
/******************************************************************************************/
function thematicMap(mapId,groupNameForData,quartierId,varToMap,direction,timeframe,classMethod,nrClasses,hue,val,prefDat,prefGeom,legendGroup) {
	this.mapId = mapId;
	this.groupNameForData = groupNameForData;
	this.quartierId = quartierId;
	this.varToMap = varToMap;
	this.direction = direction; //possible values: "weg", "ein", "total", "saldo"
	this.timeframe = timeframe;
	this.classMethod = classMethod;
	this.nrClasses = nrClasses;
	this.hue = hue;
	this.val = val;
	this.prefDat = prefDat;		//kartdat_ or quaddat_
	this.prefGeom = prefGeom;	//units_ or quadrat_
	this.legendGroup = legendGroup;//legendMap or legendDiagr
	
	this.dataArrayVar = null;
	this.dataArrayBev = null;
	this.feld_num = null;
	this.dataVarMin = null;
	this.dataVarMax = null;
	this.nrElements = null;
	this.data_loaded = false;
	this.legendState = false;
}

function getXMLData(url,callBackFunction) {
	this.url = url;
	this.callBackFunction = callBackFunction;
	this.xmlRequest = null;
} 

getXMLData.prototype.getData = function() {
	//call getURL() if available
	if (window.getURL) {
		getURL(this.url,this);
	}
	//or call XMLHttpRequest() if available
	else if (window.XMLHttpRequest) {
		var _this = this;
		this.xmlRequest = new XMLHttpRequest();
		this.xmlRequest.overrideMimeType("text/xml");
		this.xmlRequest.open("GET",this.url,true);
		this.xmlRequest.onreadystatechange = function() {_this.handleEvent()};
		this.xmlRequest.send(null);
	}
	//write an error message if neither method is available
	else {
		alert("your browser/svg viewer neither supports window.getURL nor window.XMLHttpRequest!");
	}	
}

//this is the callback method for the getURL function
getXMLData.prototype.operationComplete = function(data) {
	//check if data has a success property
	if (data.success) {
		//parse content of the XML format to the variable "node"
		var node = parseXML(data.content,document);
		this.callBackFunction(node.firstChild);
	}
	else {
		alert("something went wrong with dynamic loading of geometry!");
	}
}

//this method receives data from XMLHttpRequest
getXMLData.prototype.handleEvent = function() {
	if (this.xmlRequest.readyState == 4) {
		var importedNode = document.importNode(this.xmlRequest.responseXML.documentElement,true);
		this.callBackFunction(importedNode);
	}	
}

// workaround for firefox bug (browser cannot execute function getElementById if it is a foreign element)
// thanks to Frank Bruder for this function
function myGetElementById(id) 
{
  var element = document.getElementById(id);
  if(element==null && document.evaluate!=null) 
  {
    element = document.evaluate(
    '//*[@xml:id="' + id + '" or @id="' + id + '"]', document, function(ns) 
    {
        switch(ns) {
            case "xml": return "http://www.w3.org/XML/1998/namespace";
            case null: return null;
        }
   }, XPathResult.ANY_TYPE, null).iterateNext();
 }
 return element;
}

thematicMap.prototype.getData = function() {
	var myUrl = new getXMLData("daten/"+this.prefDat+ this.quartierId + ".xml",addKartdat);
	myUrl.getData();
}

thematicMap.prototype.readData = function() {
	var kartdatGroup = myGetElementById(this.prefDat+"quartid_"+this.quartierId);
	this.feld_num = parseInt(kartdatGroup.getAttributeNS(null,"feldnum"));
               
                var child = kartdatGroup.firstChild;
	var myWegVar;
	var myEinVar;
	var myVar;
	
	this.dataArrayVar = null; 
	this.dataArrayBev = null; 
	this.dataArrayVar = new Array(); 
	this.dataArrayBev = new Array();
	
	while (child != null) {
		if (child.nodeName == "at:dissolve_id") {
			var unitsId = child.getAttributeNS(null,"id");
			var myBevolk = parseInt(child.getAttributeNS(null,"bev_an"));
			this.dataArrayBev[unitsId] = {key:unitsId,value:myBevolk};
			this.dataArrayBev.push(this.dataArrayBev[unitsId]);
			var innerChild = child.firstChild;
			while (innerChild != null) {
				if (innerChild.nodeName == "at:weg" || innerChild.nodeName == "at:ein") {
					var innerstChild = innerChild.firstChild;
					while (innerstChild != null) {
						if (innerstChild.nodeName == "at:" + this.timeframe) {
							if (innerChild.nodeName == "at:weg") {
								myWegVar = innerstChild.getAttributeNS(null,this.varToMap);
							}
							else {
								myEinVar = innerstChild.getAttributeNS(null,this.varToMap);
							}
						}
						innerstChild = innerstChild.nextSibling;
					}
				}
				innerChild = innerChild.nextSibling;				
			}
			switch(this.direction) {
				case "weg":
					myVar = parseInt(myWegVar);
					break;
				case "ein":
					myVar = parseInt(myEinVar);
					break;
				case "total":
					myVar = parseInt(myWegVar) + parseInt(myEinVar);
					break;
				case "saldo":
					myVar = parseInt(myEinVar) - parseInt(myWegVar);
					break;
			}
			myVar = myVar / myBevolk; //normalize with population total
			this.dataArrayVar[unitsId] = {key:unitsId,value:myVar};	
			this.dataArrayVar.push(this.dataArrayVar[unitsId]);
		}
		child = child.nextSibling;
	}
	this.dataArrayVar.sort(mySortByValAsc);
	this.drawMap();
}

thematicMap.prototype.drawMap = function() {
	this.dataVarMin = this.dataArrayVar[0].value;
	this.dataVarMax = this.dataArrayVar[this.dataArrayVar.length-1].value;
	//alert(this.dataArrayVar[this.dataArrayVar.length-1].key);
	//calc colorValues
	var myColorArray = null;	
	var myColorArray = new Array();
	var sat = 0.16;
	//var val = 1;
	for (i=0;i<this.nrClasses;i++) {
		rgbArr = hsv2rgb(this.hue,sat,this.val);
		myColorArray.push(rgbArr);
		sat = sat + 1 / (this.nrClasses - 1);
		//val = val - .15 / (this.nrClasses - 1);	//to change contrast
	}	
	//calc Threshold values for classes
	var threshArray = null;
	var threshArray = new Array();
	//method equalDist
	if (this.classMethod == "equalDist") {
		var threshStep = (this.dataVarMax - this.dataVarMin) / this.nrClasses;
		var threshTemp = this.dataVarMin;
		for (i=0;i<this.nrClasses;i++) {
			threshTemp = threshTemp + threshStep;
			threshArray.push(threshTemp);
		}
	}
	//method quantiles 
	var newNrClasses = this.nrClasses;
	if (this.classMethod == "quantile") {
		if(this.direction=="saldo"){
			var classCounter = 1;
			var nrPerClass = Math.floor(this.dataArrayVar.length / this.nrClasses);
			var modVal = this.dataArrayVar.length % this.nrClasses;
			var plusVal = 0;
			if (modVal > 0) {
				plusVal = 1;
			}
			var tempThresh = nrPerClass + plusVal
			for (i=0;i<this.dataArrayVar.length;i++) {
				if (i >= tempThresh) {
					threshArray.push(this.dataArrayVar[i].value);
					if (plusVal == 1) {
						if (modVal > 1) {
							modVal--;
						}
						else {
							plusVal = 0;
						}	
					}
					classCounter += 1;
					tempThresh += nrPerClass + plusVal;
				}
			}
			threshArray.push(this.dataVarMax)
		}else{		
			j=0;
			for (i=0;i<this.dataArrayVar.length;i++) {
				if(this.dataArrayVar[i].value==0){
			  		j++;
					//this.dataArrayVar.shift();
	
				}
			}
			//alert(j);
			var nrUnequalNull = this.dataArrayVar.length-j;
					
			if (nrUnequalNull < newNrClasses - 1){
				newNrClasses = nrUnequalNull + 1 ;
				alert("Klassenanzahl eingeschraenkt, weil nicht mehr als "+ nrUnequalNull + "\nGemeinden oder Quartiere Werte ueber null haben.")
				}
				
			var nrPerClass = Math.floor((this.dataArrayVar.length-j) / (newNrClasses-1));
	
			var modVal = (this.dataArrayVar.length-j) % (newNrClasses-1);	// Restwert 
			
			var plusVal = 0;
			if (modVal > 0) {
				plusVal = 1;
			}
			var tempThresh = j + nrPerClass + plusVal	//wenn es nen rest gibt, dann wird die anzahl der eintraege pro klasse um eines erhoeht
			//alert(this.dataArrayVar[j].value);
			threshArray.push(this.dataArrayVar[j].value);
			for (i=j;i<this.dataArrayVar.length;i++) {
	
				if (i >= tempThresh) {
					threshArray.push(this.dataArrayVar[i].value);
					//alert("Grenze" + i +": "+ this.dataArrayVar[i].value);
					if (plusVal == 1) {
						if (modVal > 1) {
							modVal--;
						}
						else {
							plusVal = 0;
						}	
					}
					tempThresh += nrPerClass + plusVal;
	
			}
	
				
			}
			threshArray.push(this.dataVarMax);
			//alert("GrenzeMax:" + this.dataVarMax);
		}
	}
	//color Map 
	var colIndex = 0; 
	var unitsId = ""; 
	for (i=0;i<this.dataArrayVar.length;i++) { 
		//hier wurde = Zeichen eingefuegt
		if (this.dataArrayVar[i].value >= threshArray[colIndex]) { 
			//change class and color 
			while (this.dataArrayVar[i].value >= threshArray[colIndex]) {
				colIndex += 1;
			}
			if (colIndex > myColorArray.length - 1) {
				colIndex = myColorArray.length - 1;
			}			
		} 
		unitsId =this.prefGeom+ this.dataArrayVar[i].key.replace(this.prefDat,"");
		//alert(unitsId);
		myUnit = document.getElementById(unitsId); 
		myUnit.setAttributeNS(null,"fill","rgb("+myColorArray[colIndex]["red"]+","+myColorArray[colIndex]["green"]+","+myColorArray[colIndex]["blue"]+")"); 
		myUnit.setAttributeNS("http://www.carto.net/attrib/","Class",colIndex+1);
		myUnit.setAttributeNS("http://www.carto.net/attrib/","Value",this.dataArrayVar[i].value);
	}	
	//draw legend			
	colIndex = 0; 
	var yOffset = 10;
	if(this.legendState==true){
		var node = document.getElementById(this.legendGroup+"_g");
		document.getElementById(this.legendGroup).removeChild(node);
	}
	
	var groupe = document.createElementNS(svgNS,"g");
	groupe.setAttributeNS(null,"id",this.legendGroup+"_g");	
	var node = document.createElementNS(svgNS,"rect");
	node.setAttributeNS(null,"id",this.legendGroup+"_class_0");
	node.setAttributeNS(null,"x","0");
	node.setAttributeNS(null,"y",yOffset);
	node.setAttributeNS(null,"width","20");
	node.setAttributeNS(null,"height","10");
	node.setAttributeNS(null,"style","fill:rgb("+myColorArray[colIndex]["red"]+","+myColorArray[colIndex]["green"]+","+myColorArray[colIndex]["blue"]+");stroke:rgb(102,102,102);stroke-linecap:round;stroke-linejoin:round;"); 
	
	groupe.appendChild(node);	
	var node = document.createElementNS(svgNS,"text");
	node.setAttributeNS(null,"id",this.legendGroup+"_classTxt_0");
	node.setAttributeNS(null,"x","30");
	node.setAttributeNS(null,"y",yOffset+10);
	node.setAttributeNS(null,"pointer-events","none");
	if ((this.classMethod == "quantile")&&(this.direction!="saldo")) {
		var selectionText = document.createTextNode("0%");
	} else {
		var selectionText = document.createTextNode(Math.round(this.dataVarMin*1000)/10+"% bis "+Math.round(threshArray[colIndex]*1000)/10+"%"); //+", Kl. "+eval(colIndex+1)

	}
	node.appendChild(selectionText);
	groupe.appendChild(node);	
	yOffset = yOffset+20;			
	for (i=0;i<newNrClasses;i++) {
		if (colIndex != newNrClasses-1) { 
			colIndex += 1; 		
			var node = document.createElementNS(svgNS,"rect");
			node.setAttributeNS(null,"id",this.legendGroup+"_class_"+colIndex);
			node.setAttributeNS(null,"x","0");
			node.setAttributeNS(null,"y",yOffset);
			node.setAttributeNS(null,"width","20");
			node.setAttributeNS(null,"height","10");
			node.setAttributeNS(null,"style","fill:rgb("+myColorArray[colIndex]["red"]+","+myColorArray[colIndex]["green"]+","+myColorArray[colIndex]["blue"]+");stroke:rgb(102,102,102);stroke-linecap:round;stroke-linejoin:round;"); 
			groupe.appendChild(node);
			
			var node = document.createElementNS(svgNS,"text");
			node.setAttributeNS(null,"id",this.legendGroup+"_classTxt_"+colIndex);
			node.setAttributeNS(null,"x","30");
			node.setAttributeNS(null,"y",yOffset+10);
			node.setAttributeNS(null,"pointer-events","none");
			var selectionText = document.createTextNode(Math.round(threshArray[colIndex-1]*1000)/10+"% bis "+Math.round(threshArray[colIndex]*1000)/10+"%");	//+", Kl. "+eval(colIndex+1)
			node.appendChild(selectionText);
			groupe.appendChild(node);
			
			yOffset = yOffset + 22;
			this.legendState = true;
		} 
	}
	document.getElementById(this.legendGroup).appendChild(groupe);
	//document.getElementById(this.legendGroup+"_txt").setAttribute("y",yOffset+10);
	var heightLegend = newNrClasses*24-(newNrClasses*1.4);
	if(heightLegend < 100) {
		heightLegend = 100;
		
	}
	document.getElementById(this.legendGroup+"_rect").setAttribute("height",heightLegend);
}

function mySortByValAsc(a,b) {
	if (a.value > b.value) {
		return 1;
	}
	else {
		return -1;
	}
}


function addKartdat(data) {
		var dataLayerId = data.getAttributeNS(null,"id").replace(/_temp/,"");
		var dataGroup = document.getElementById(dataLayerId + "_data");
		if (thematicMaps[dataLayerId].data_loaded == true) {
			dataGroup.removeChild(dataGroup.firstChild);
		}
		dataGroup.appendChild(data);
		thematicMaps[dataLayerId].data_loaded = true;
		thematicMaps[dataLayerId].readData();
}

function debugAlert(debugText) {
	//case linux/safari/opera
	//browserEval("alert('"+debugText+"')");
	//other
	alert(debugText);
}
